﻿param([Int]$rowcount, [Int]$days = 1, [String]$logpath=(Resolve-Path .\).Path, [switch]$sendToAPI, [String]$token, [String]$name)

$logDirectory = $logpath

$uri = "https://services.qubedocs.com/Generate/UploadLogs"

$path = (Resolve-Path .\).Path

# Delete old package from root
Remove-Item "QDLogsPackage.7z" -ErrorAction Ignore

Set-Location -Path $logDirectory

$startDate = (Get-Date).AddDays(-$days)

# Delete old package from log folder
Remove-Item "QDLogsPackage.7z" -ErrorAction Ignore
Remove-Item "QDTemptm1server.log" -ErrorAction Ignore

Function GetDateFromLine([String]$line){
    try{
        $mydate = ($line -split '\s+|\t+')[3]
        $myLogDate = [datetime]::ParseExact($mydate, "yyyy-MM-dd", $null)
        Return $myLogDate;
    }Catch{
        Return ""
    }
}

Function FileRowCount($file){

    @(100) |% {
    $count = 0
    Get-Content -Path $file -ReadCount $_ |% { $count += $_.Count }
    }
    
    Return $count
}

Function ProcessFileDates{
    Param([DateTime]$startDate, [String]$filePath)

     "Checking: "+ $filePath
     $lastLine = Get-Content -Path $filePath -Tail 1
     $logEndDate=GetDateFromLine($lastLine)
     if($startDate -ilt $logEndDate){
        Get-Content $filePath | ? {
             
               $LogDate = GetDateFromLine($_)
             
              $logdate -ige $startDate
            } | Add-Content -Path .\QDTemptm1server.log
     }
    #$startDate
}


Function ProcessFileCounts{
    Param($linesLeft, [String]$filePath)
    "Checking: "+ $filePath
    Get-Content -Path $filePath -Tail $linesLeft | Add-Content -Path .\QDTemptm1server.log 
}


$fileList = Get-Item tm1server*.*log* | Sort-Object -Property Name 

if($fileList.Count -gt 0){
    $linesLeft = $rowcount

    For ($i=0; $i -le $fileList.Count-1; $i++) {
        if($rowcount -gt 0){
             if($linesLeft -gt 0){
                #$fileList[$i].GetType().FullName
                $lineCount=FileRowCount($fileList[$i])
                ProcessFileCounts $linesLeft  $fileList[$i].FullName
                $linesLeft-=$lineCount
             }else{
                
             }

        }else{

            ProcessFileDates $startDate $fileList[$i].FullName
        }
    }
}

# Zipping up tm1server.log
   
    #If (-not (Test-Path "$path\7za.exe")) {throw "$path\7za.exe needed"} 
    #Set-Alias sz "$path\7za.exe"  

	If (-not (Test-Path "$path\QUBEdocsFiles\7za.exe")) {throw "$path\QUBEdocsFiles\7za.exe needed"} 
	Set-Alias sz "$path\QUBEdocsFiles\7za.exe"       
    
    if(Test-Path .\QDTemptm1server.log){
       sz a -mx9 QDLogsPackage.7z (ls QDTemptm1server.log)
       sz rn QDLogsPackage.7z QDTemptm1server.log tm1server.log
    }
   

    #take all process files
        Get-Item TM1processerror*.log | 
            Foreach-Object {
    
            $ts = ((($_.Name -split '_')[1]).ToCharArray() | select -First 8) -join ""
            $processdate = [datetime]::ParseExact($ts, "yyyyMMdd", $null)
    
            if($processdate -ige $startDate) {
    
               "Packing up :"+$_.Name
	            sz a -mx9 QDLogsPackage.7z (ls $_.Name) 
               
            }
        }



if(Test-Path .\QDLogsPackage.7z){ 
 if($sendToAPI){   
    "Sending..."
	[Net.ServicePointManager]::SecurityProtocol = [Net.SecurityProtocolType]::Tls12

    # https://stackoverflow.com/questions/25075010/upload-multiple-files-from-powershell-script
	
	[Environment]::CurrentDirectory = (Get-Location -PSProvider FileSystem).ProviderPath # sync execution path for script

    $fileBin = [IO.File]::ReadAllBytes($logDirectory + "\QDLogsPackage.7z")

    $enc = [System.Text.Encoding]::GetEncoding("iso-8859-1")
    $fileEnc = $enc.GetString($fileBin)
    $boundary = [System.Guid]::NewGuid().ToString()   

    $LF = "`n"
    $bodyLines = (
        "--$boundary",
        "Content-Disposition: form-data; filename=`"QDLogsPackage.7z`"; name=`"Package`";",
	    "Content-Type: application/x-7z-compressed $LF",
        $fileEnc,
        "--$boundary",
        "Content-Disposition: form-data;  name=`"Token`"$LF",
        $token,
	    "--$boundary",
        "Content-Disposition: form-data;  name=`"Name`"$LF",
        $name,
        "--$boundary--$LF"
    ) -join $LF

    try {
        Invoke-RestMethod -Uri $uri -Method Post -ContentType "multipart/form-data; boundary=`"$boundary`"" -TimeoutSec 60 -Body $bodyLines
    }
    catch [System.Net.WebException] {
        Write-Error( "FAILED to reach '$URL': $_" )
        throw $_
    }
    }else{
        #if moved to root folder
        #if($path -ne $logDirectory){
        
          #  Remove-Item $path\QDLogsPackage.7z -ErrorAction Ignore
          #  Move-Item .\QDLogsPackage.7z $path
        #}
    }

}

else{
    "No logs found"
    }

#Clean up
#remove this if sending to API

#If moved to root folder
#if($path -ne $logDirectory){
  #  Remove-Item "QDLogsPackage.7z" -ErrorAction Ignore
#}
if($sendToAPI){
    Remove-Item "QDLogsPackage.7z" -ErrorAction Ignore
}
Remove-Item "QDTemptm1server.log" -ErrorAction Ignore

   "Finished"

Set-Location -Path $path